@@ -166,7 +166,7 @@ gdk_drag_context_get_source_window (GdkDragContext *context)
  * gdk_drag_context_get_dest_window:
  * @context: a #GdkDragContext
  *
- * Returns the destination windw for the DND operation.
+ * Returns the destination window for the DND operation.
  *
  * Returns: (transfer none): a #GdkWindow
  *
@@ -184,7 +184,7 @@ gdk_drag_context_get_dest_window (GdkDragContext *context)
  * gdk_drag_context_get_protocol:
  * @context: a #GdkDragContext
  *
- * Returns the drag protocol thats used by this context.
+ * Returns the drag protocol that is used by this context.
  *
  * Returns: the drag protocol
  *
@@ -289,7 +289,7 @@ gdk_drag_context_class_init (GdkDragContextClass *klass)
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (GdkDragContextClass, cancel),
                   NULL, NULL,
-                  g_cclosure_marshal_VOID__ENUM,
+                  NULL,
                   G_TYPE_NONE, 1, GDK_TYPE_DRAG_CANCEL_REASON);
 
   /**
@@ -311,7 +311,7 @@ gdk_drag_context_class_init (GdkDragContextClass *klass)
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (GdkDragContextClass, drop_performed),
                   NULL, NULL,
-                  g_cclosure_marshal_VOID__INT,
+                  NULL,
                   G_TYPE_NONE, 1, G_TYPE_INT);
 
   /**
@@ -334,7 +334,7 @@ gdk_drag_context_class_init (GdkDragContextClass *klass)
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (GdkDragContextClass, dnd_finished),
                   NULL, NULL,
-                  g_cclosure_marshal_VOID__VOID,
+                  NULL,
                   G_TYPE_NONE, 0);
 
   /**
@@ -356,7 +356,7 @@ gdk_drag_context_class_init (GdkDragContextClass *klass)
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (GdkDragContextClass, action_changed),
                   NULL, NULL,
-                  g_cclosure_marshal_VOID__FLAGS,
+                  NULL,
                   G_TYPE_NONE, 1, GDK_TYPE_DRAG_ACTION);
 }
 
