@@ -25,6 +25,7 @@
 #include <config.h>
 #include <stdint.h>
 #include <wayland-client.h>
+#include <wayland-cursor.h>
 #include <wayland-egl.h>
 #include <gdk/wayland/tablet-unstable-v2-client-protocol.h>
 #include <gdk/wayland/gtk-shell-client-protocol.h>
@@ -38,7 +39,6 @@
 #ifdef HAVE_XDG_ACTIVATION
 #include <gdk/wayland/xdg-activation-v1-client-protocol.h>
 #endif
-#include <gdk/wayland/cursor/wayland-cursor.h>
 
 #include <glib.h>
 #include <gdk/gdkkeys.h>
@@ -52,6 +52,9 @@
 
 G_BEGIN_DECLS
 
+#define GDK_WAYLAND_MAX_THEME_SCALE 4
+#define GDK_WAYLAND_THEME_SCALES_COUNT GDK_WAYLAND_MAX_THEME_SCALE
+
 #define GDK_ZWP_POINTER_GESTURES_V1_VERSION 1
 
 typedef struct _GdkWaylandSelection GdkWaylandSelection;
@@ -116,7 +119,7 @@ struct _GdkWaylandDisplay
 
   GList *current_popups;
 
-  struct wl_cursor_theme *cursor_theme;
+  struct wl_cursor_theme *scaled_cursor_themes[GDK_WAYLAND_THEME_SCALES_COUNT];
   gchar *cursor_theme_name;
   int cursor_theme_size;
   GHashTable *cursor_cache;
